@@ -1,6 +1,6 @@
 import unittest
 import json
-from flask import Flask, Blueprint, redirect, views
+from flask import Flask, Blueprint, redirect, views, abort as flask_abort
 from flask.signals import got_request_exception, signals_available
 try:
     from mock import Mock
@@ -10,6 +10,7 @@ except:
 import flask
 import werkzeug
 from werkzeug.exceptions import HTTPException, Unauthorized, BadRequest, NotFound
+from werkzeug.http import quote_etag, unquote_etag
 from flask_restful.utils import http_status_message, unpack
 import flask_restful
 import flask_restful.fields
@@ -52,7 +53,7 @@ class APITestCase(unittest.TestCase):
         response.headers = {}
         with app.test_request_context('/foo'):
             response = api.unauthorized(response)
-        assert_false('WWW-Autheneticate' in response.headers)
+        assert_false('WWW-Authenticate' in response.headers)
 
     def test_unauthorized(self):
         app = Flask(__name__)
@@ -779,9 +780,55 @@ class APITestCase(unittest.TestCase):
         resp = app.post('/ids/3')
         self.assertEquals(resp.status_code, 405)
         self.assertEquals(resp.content_type, api.default_mediatype)
-        self.assertEquals(set(resp.headers.get_all('Allow')),
+        # Allow can be of the form 'GET, PUT, POST'
+        allow = ', '.join(set(resp.headers.get_all('Allow')))
+        allow = set(method.strip() for method in allow.split(','))
+        self.assertEquals(allow,
                           set(['HEAD', 'OPTIONS'] + HelloWorld.methods))
 
+    def test_exception_header_forwarded(self):
+        """Test that HTTPException's headers are extended properly"""
+        app = Flask(__name__)
+        app.config['DEBUG'] = True
+        api = flask_restful.Api(app)
+
+        class NotModified(HTTPException):
+            code = 304
+
+            def __init__(self, etag, *args, **kwargs):
+                super(NotModified, self).__init__(*args, **kwargs)
+                self.etag = quote_etag(etag)
+
+            def get_headers(self, *args, **kwargs):
+                """Get a list of headers."""
+                return [('ETag', self.etag)]
+
+        class Foo1(flask_restful.Resource):
+            def get(self):
+                flask_abort(304, etag='myETag')
+
+        api.add_resource(Foo1, '/foo')
+        flask_abort.mapping.update({304: NotModified})
+
+        with app.test_client() as client:
+            foo = client.get('/foo')
+            self.assertEquals(foo.get_etag(),
+                              unquote_etag(quote_etag('myETag')))
+
+    def test_exception_header_forwarding_doesnt_duplicate_headers(self):
+        """Test that HTTPException's headers do not add a duplicate
+        Content-Length header
+
+        https://github.com/flask-restful/flask-restful/issues/534
+        """
+        app = Flask(__name__)
+        api = flask_restful.Api(app)
+
+        with app.test_request_context('/'):
+            r = api.handle_error(BadRequest())
+
+        self.assertEqual(len(r.headers.getlist('Content-Length')), 1)
+
     def test_will_prettyprint_json_in_debug_mode(self):
         app = Flask(__name__)
         app.config['DEBUG'] = True
